package com.bawei.es.server.impl;

import com.bawei.es.domain.*;
import com.bawei.es.server.EsDocumentServer;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.delete.DeleteRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.update.UpdateRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.indices.GetIndexRequest;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.xcontent.XContentFactory;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
@AllArgsConstructor
@Slf4j
public class EsDocumentServerImpl implements EsDocumentServer {
    private RestHighLevelClient restHighLevelClient;
    @Override
    public void add(DocumentReq documentReq) {
        //打印请求参数
        log.info("参数为{}",documentReq);
        //效验索引是否存在
        if (findIndexName(documentReq.getIndexName())){
            //批量添加
            BulkRequest bulkRequest = new BulkRequest();
            List<Map<String, Object>> data = documentReq.getDocumentContent();
            //判断数据是否为空
            if (data==null||data.size()==0){
                throw new RuntimeException("数据为空");
            }
            //循环集合
            for (Map<String, Object> datum : data) {
                //创建索引请求indexRequest
                IndexRequest indexRequest = new IndexRequest(documentReq.getIndexName());
                //遍历数据
                log.info("数据{}",datum);
                if (documentReq.getIdField()!=null&&datum.containsKey(documentReq.getIdField())){
                    //获取data中的map取id
                    String uuid = datum.get(documentReq.getIdField()).toString();
                    //判断isNeedId是否==true和uuid非空验证
                    if (documentReq.isNeedId()==true&&uuid!=null){
                        //设置id
                        indexRequest.id(uuid);
                    }
                }
                //设置内容
                indexRequest.source(datum);
                bulkRequest.add(indexRequest);
            }
            try {
                restHighLevelClient.bulk(bulkRequest,RequestOptions.DEFAULT);
            } catch (Exception e) {
                log.info("es插入失败异常信息{}",e);
                throw new RuntimeException("es插入失败");
            }
        }else {
            throw new RuntimeException("索引名不存在");
        }


    }

    @Override
    public void del(DocumentDelReq documentDelReq) {
        if (findIndexName(documentDelReq.getIndexName())){
            BulkRequest bulkRequest = new BulkRequest();
            try {
                for (String id : documentDelReq.getIds()) {
                    DeleteRequest deleteRequest = new DeleteRequest(documentDelReq.getIndexName(), id);
                    bulkRequest.add(deleteRequest);
                }
                restHighLevelClient.bulk(bulkRequest,RequestOptions.DEFAULT);
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }else {
            throw new RuntimeException("索引名不存在");
        }
    }

    @Override
    public void upd(DocumentUpdReq documentReq) {
        log.info("打印请求{}",documentReq);
        if (findIndexName(documentReq.getIndexName())){
            String name = documentReq.getDocument().getName();
            Integer age = documentReq.getDocument().getAge();
            try {
                UpdateRequest updateRequest = new UpdateRequest();
                updateRequest.id(documentReq.getId());
                updateRequest.index(documentReq.getIndexName());
                updateRequest.doc(
                        XContentFactory.jsonBuilder().startObject()
                        .field("name",name)
                        .field("age",age)
                        .endObject()
                );
                restHighLevelClient.update(updateRequest,RequestOptions.DEFAULT);
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }else {
            throw new RuntimeException("索引名不存在");
        }
    }

    @Override
    public ArrayList<Map<String, Object>> list(DocumentListReq documentListReq) {
        //返回结果
        ArrayList<Map<String, Object>> maps = new ArrayList<>();
        //输出请求参数
        log.info("打印请求参数{}",documentListReq);
        //判断索引是否存在
        if (findIndexName(documentListReq.getIndexName())){
            // 搜索源构建对象
            SearchRequest searchRequest = new SearchRequest(documentListReq.getIndexName());
            // 搜索源构建对象
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            // 搜索方式
            BoolQueryBuilder boolQuery = QueryBuilders.boolQuery();
            //全文检索
            if(documentListReq.getKeyWord()!=null&&!documentListReq.getKeyWord().equals("")){
                boolQuery.must(QueryBuilders.matchQuery(documentListReq.getQueryFields().get(0),documentListReq.getKeyWord()));
            }
            searchSourceBuilder.query(boolQuery);
            //判断分页参数是否为空
            if(documentListReq.getBasePage().getPageNum()!=null&&documentListReq.getBasePage().getPageSize()!=null){
                searchSourceBuilder.from((documentListReq.getBasePage().getPageNum()-1)*documentListReq.getBasePage().getPageSize());
                searchSourceBuilder.size(documentListReq.getBasePage().getPageSize());
            }
            //检索字段
            if (documentListReq.getQueryFields()!=null&&!documentListReq.getQueryFields().equals("")){
                String[] array = documentListReq.getReturnFields().toArray(new String[0]);
                searchSourceBuilder.fetchSource(array,null);
            }
            try {
                //向es发送请求
                SearchResponse search = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
                SearchHits hits = search.getHits();
                //循环查询结果
                SearchHit[] hits1 = hits.getHits();
                for (SearchHit documentFields : hits1) {
                    //转型
                    Map<String, Object> sourceAsMap = documentFields.getSourceAsMap();
                    log.info("遍历结果{}",sourceAsMap);
                    maps.add(sourceAsMap);
                }
                return maps;
            } catch (Exception e) {
                throw new RuntimeException(e);
            }

        }else {
            throw new RuntimeException("索引名不存在");
        }
    }


    public boolean findIndexName(String indexName){
        if (indexName!=null){
            try {
                GetIndexRequest getIndexRequest = new GetIndexRequest(indexName);
                return restHighLevelClient.indices().exists(getIndexRequest, RequestOptions.DEFAULT);
            } catch (Exception e) {
                log.info("查询异常信息{}",e);
                throw new RuntimeException("索引查询失败");
            }
        }else {
            throw new  RuntimeException("索引名为空");
        }
    }
}
